#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"
#include "xscugic.h" //for interrupts
#include "xgpio.h"
#include "partest.h"
#include "portmacro.h"
#include "serial.h"
#include "xaxidma.h"
#include "iic.h"
#include "audio.h"
#include "dma.h"
#include "lh_main.h"
#include "xtime_l.h"
#include <math.h>

#define INTERRUPT_DEVICE_ID		XPAR_PS7_SCUGIC_0_DEVICE_ID
#define gpioDEVICE_ID			XPAR_AXI_GPIO_0_DEVICE_ID
#define sensorDEVICE_ID			XPAR_AXI_GPIO_1_DEVICE_ID
#define gpioINTC_INTR_ID 		XPAR_FABRIC_AXI_GPIO_0_IP2INTC_IRPT_INTR
#define sensorINTC_INTR_ID		XPAR_FABRIC_AXI_GPIO_1_IP2INTC_IRPT_INTR
#define gpioBTN_INTR_MASK		XGPIO_IR_MASK

static xSemaphoreHandle btn_sem;
static xSemaphoreHandle uart_sem;
static xSemaphoreHandle audio_sem;
static xSemaphoreHandle calc_sem;
static xSemaphoreHandle sensor_sem;

static XIic sIic;
static XAxiDma sAxiDma;		/* Instance of the XAxiDma */

static XGpio xBtnInstance;
static XGpio xSensorInstance;
xComPortHandle xPort = 0;
extern XScuGic xInterruptController;

#define NUM_BUFS	2
static int32_t buffer[NUM_BUFS][BUFFER_SIZE];
static int buf_num = 0;

static sound_t sounds[NUM_STRINGS];

static double decay_rates[15] = {			-0.0002, -0.00035 -0.0005,
											-0.00075, -0.001,  -0.0015, -0.002,  -0.0035,
											-0.005,   -0.0075, -0.01,   -0.02,   -0.04,
											-0.075,   -0.1};

static int sensor_map[NUM_STRINGS] =   {8, 6, 10, 0, 4, 2, 3, 5, 7, 1, 11, 9};

#define NUM_SCALES	5
static scale_t scales[NUM_SCALES] =
{
		{
			.length = 7,
			.intervals = {2, 2, 1, 2, 2, 2, 1},
			.name = "Major"
		},
		{
			.length = 11,
			.intervals = {1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
			.name = "Chromatic"
		},
		{
			.length = 7,
			.intervals = {2, 1, 2, 2, 1, 2, 2},
			.name = "Minor"
		},
		{
			.length = 5,
			.intervals = {3, 2, 2, 3, 2},
			.name = "Minor Pentatonic"
		},
		{
			.length = 6,
			.intervals = {3, 2, 1, 1, 3, 2},
			.name = "Blues"
		}
};
static char * str_notes[NUM_NOTES] = {"C", "Db", "D", "Eb", "F", "Gb", "G", "Ab", "A", "Bb", "B", "Cb"};

static menu_t main_menu, tuning_menu, tone_menu;
static char ansi_code[2];

static table_t table;

void dma_mm2s_ISR(void *  XAxiDma_instance);
void dma_s2mm_ISR(void *  XAxiDma_instance);
static void _tune(int root, int scale_num);

static const ivt_t _ivt[] = {
	//IIC
	{XPAR_FABRIC_AXI_IIC_0_IIC2INTC_IRPT_INTR, (Xil_ExceptionHandler)XIic_InterruptHandler, &sIic},
	//DMA Stream to MemoryMap Interrupt handler
	{XPAR_FABRIC_AXI_DMA_0_S2MM_INTROUT_INTR, (Xil_ExceptionHandler)dma_s2mm_ISR, &sAxiDma},
	//DMA MemoryMap to Stream Interrupt handler
	{XPAR_FABRIC_AXI_DMA_0_MM2S_INTROUT_INTR, (Xil_ExceptionHandler)dma_mm2s_ISR, &sAxiDma},
	//User I/O (buttons, switches, LEDs)
	//{XPAR_FABRIC_AXI_GPIO_0_IP2INTC_IRPT_INTR, (Xil_ExceptionHandler)fnUserIOIsr, &sUserIO}
};

#define MS_TO_TICKS(ms) (ms / portTICK_PERIOD_MS)

const TickType_t delay = MS_TO_TICKS(500);

static void _print(char *str)
{
	xSemaphoreTake(uart_sem, portMAX_DELAY);
	vSerialPutString( xPort, str, strlen(str));
	xSemaphoreGive(uart_sem);
}

static void _print_char(char c)
{
	char buf[2];
	buf[0] = c;
	buf[1] = 0;
	_print(buf);
}

static char _read()
{
	char c;
	xSemaphoreTake(uart_sem, portMAX_DELAY);
	xSerialGetChar(xPort, &c, portMAX_DELAY);
	xSemaphoreGive(uart_sem);
	return c;
}

static void _create_sine_tables()
{
	int i, j;
	for(i = 0; i < NUM_STRINGS; i++)
	{
		table.length[i] = (uint16_t)(AUDIO_SAMPLING_RATE / base_freqs[sounds[i].note]); //length = samples / cycle
		for(j=0; j< table.length[i]; j++)
		{
			table.value[i][j] = (int32_t)(sin((2*PI*j * pow(2, table.octave[i])) / table.length[i]) * 150);
		}
	}
}

static void _create_decay_tables()
{
	int i, j;
	for(i = 0; i < NUM_HARMONICS; i++)
	{
		for(j=0; j< MAX_DECAY; j++)
		{
			table.decay[i][j] = (uint16_t)(exp(j * decay_rates[table.decay_const_index[i]]) * 80);
		}
	}
}

static void _audio_task()
{
	Xil_Out32(I2S_TRANSFER_CONTROL_REG, 0x00000000); //disable transfer to begin with, transfer gets enabled in audioplay function
	Xil_Out32(I2S_TRANSFER_CONTROL_REG, 0x00000001);
	while(1)
	{
		xSemaphoreTake(audio_sem, portMAX_DELAY);
		fnAudioPlay(sAxiDma,BUFFER_SIZE, buffer[buf_num]);
		xSemaphoreGive(calc_sem); // let the next calculations take place
		buf_num =(buf_num + 1) % NUM_BUFS;
	}
}

static void _init_strings()
{
	int i, j;
	for(i=0; i<NUM_STRINGS; i++)
	{
		sounds[i].is_active = 0;
		sounds[i].note = i;
		table.octave[i] = 2;
		sounds[i].duration = MAX_DECAY * DECAY_TIME_SCALE;//AUDIO_SAMPLING_RATE * 4;
		for(j = 0; j < NUM_HARMONICS; j++)
		{
			sounds[i].scale[j] = 0;
		}
		sounds[i].scale[0] = 6;
		sounds[i].scale[1] = 4;
		sounds[i].scale[2] = 3;
		sounds[i].scale[3] = 1;
		sounds[i].scale[4] = 3;
		sounds[i].scale[5] = 1;
		sounds[i].scale[6] = 4;//4;
		sounds[i].time = 0;
	}

		table.decay_const_index[0] = 4;
		table.decay_const_index[1] = 2;
		table.decay_const_index[2] = 8;
		table.decay_const_index[3] = 6;
		table.decay_const_index[4] = 10;
		table.decay_const_index[5] = 12;
		table.decay_const_index[6] = 12;

	_tune(C, 1);
	_create_sine_tables();
	_create_decay_tables();
}

static void _play_string(int string)
{
	sounds[string].is_active = 1;//!sounds[string].is_active;
	sounds[string].duration = AUDIO_SAMPLING_RATE * 4; // four seconds
	sounds[string].time = 0;
}


static void _button_task()
{
	static int str = 0;
	while(1)
	{
		xSemaphoreTake(btn_sem, portMAX_DELAY);
		_play_string(str);
		str = (str + 1) % NUM_STRINGS;
	}
}

#define SENSOR_MASK		0x00007FFF
static void _sensor_task()
{
	uint32_t status,input, prev, temp, i;
	prev = 0xFFFFFFFF;
	while(1)
	{
		xSemaphoreTake(sensor_sem, portMAX_DELAY);
		input = XGpio_DiscreteRead(&xSensorInstance, 1);
		for(i = 0; i < NUM_STRINGS; i++)
		{
			if((~input & (1 << i)))
			{
				status &= ~(1 << sensor_map[i]);
			}
			else
			{
				status |= (1 << sensor_map[i]);
			}
		}
		temp = ((status ^ prev) & prev) & SENSOR_MASK;
		prev = status;
		for(i = 0; i < NUM_STRINGS; i++)
		{
			if(temp & (1 << i))
			{
				_play_string(i);
			}
		}
	}
}

#define DEFAULT_OCTAVE  2
static void _tune(int root, int scale_num)
{
	int i, note, octave, interval;
	note = root;
	scale_t * scale;
	scale = &(scales[scale_num]);
	interval = 0;
	octave = DEFAULT_OCTAVE;
	for(i=0; i<NUM_STRINGS; i++)
	{
		sounds[i].note = note;
		table.octave[i] = octave;
		note += scale->intervals[interval];
		interval++;
		if(interval >= scale->length)
		{
			interval = 0;
		}
		if(note >= NUM_NOTES)
		{
			note = note % NUM_NOTES;
			octave++;
		}
	}
	_create_sine_tables();
}


static void _tuning_menu_fun()
{
	int i;
	char c;
	int root_note = 0;
	int scale_num = 0;

	while(1)
	{
		_print(ansi_code);
		_print("[2J");
		_print("Tuning");
		_print(ansi_code);
		_print("E");

		for(i = 0; i < tuning_menu.size; i++)
		{
			_print(tuning_menu.display[i]);
			if(i == 0) //alsoo print currently selected key
			{
				_print(" ");
				_print(str_notes[root_note]);
			}
			else if(i == 1)
			{
				_print("");
				_print(scales[scale_num].name);
			}

			if(tuning_menu.selected == i)
			{
				_print(" <--");
			}
			else
			{
				_print("      ");
			}
			_print(ansi_code);
			_print("E");
		}
		_print(ansi_code);
		_print("[H");

		c = _read();
		if((c == '\n') || (c == '\r'))
		{
			if(tuning_menu.selected == 3)
			{
				return;
			}
			else if(tuning_menu.selected == 2)
			{
				_tune(root_note, scale_num);
			}
		}
		else if(c == 's')
		{
			tuning_menu.selected++;
			if(tuning_menu.selected >= tuning_menu.size)
				tuning_menu.selected = tuning_menu.size - 1;
		}
		else if(c == 'w')
		{
			tuning_menu.selected--;
			if(tuning_menu.selected < 0)
				tuning_menu.selected = 0;
		}
		else if(c == 'd')
		{
			if(tuning_menu.selected == 0)
			{
				root_note++;
				if(root_note >= NUM_NOTES )
				{
					root_note = 0;
				}
			}
			else if(tuning_menu.selected == 1)
			{
				scale_num++;
				if(scale_num >= NUM_SCALES)
				{
					scale_num = 0;
				}
			}
		}
		else if(c == 'a')
		{
			if(tuning_menu.selected == 0)
			{
				root_note--;
				if(root_note < 0)
				{
					root_note = NUM_NOTES - 1;
				}
			}
			else if(tuning_menu.selected == 1)
			{
				scale_num--;
				if(scale_num < 0)
				{
					scale_num = NUM_SCALES -1;
				}
			}
		}
	}
}


#define MAX_HARMONIC_SCALE	10


static void _tone_menu_fun()
{
	int i, har_num;
	char c;
	har_num = 0;
	char buf[6];
	while(1)
	{
		_print(ansi_code);
		_print("[2J");
		_print("Tone");
		_print(ansi_code);
		_print("E");

		for(i = 0; i < tone_menu.size; i++)
		{
			_print(tone_menu.display[i]);
			if(i == 0) //harmonic number
			{
				if(har_num == 0)
				{
					_print("root");
				}
				else
				{
					_print_char('0' + har_num);
				}

			}
			else if(i == 1) //Value
			{
				if(sounds[0].scale[har_num] < 10)
				{
					_print_char('0' + sounds[0].scale[har_num]);
				}
				else
				{
					_print_char('1');
					_print_char('0' + sounds[0].scale[har_num] - 10);
				}
			}
			else if(i == 2)
			{
				if(table.decay_const_index[har_num] < 9)
				{
					_print_char('1' + table.decay_const_index[har_num]);
				}
				else
				{
					_print_char('1');
					_print_char('1' + table.decay_const_index[har_num] - 10);
				}
			}


			if(tone_menu.selected == i)
			{
				_print(" <--");
			}
			else
			{
				_print("      ");
			}
			_print(ansi_code);
			_print("E");
		}
		_print(ansi_code);
		_print("[H");

		c = _read();
		if((c == '\n') || (c == '\r'))
		{
			if(tone_menu.selected == tone_menu.size - 1)
			{
				return;
			}
			else if(tone_menu.selected == tone_menu.size - 2) //apply decays
			{
				_create_decay_tables();
			}
		}
		else if(c == 's')
		{
			tone_menu.selected++;
			if(tone_menu.selected >= tone_menu.size)
				tone_menu.selected = tone_menu.size - 1;
		}
		else if(c == 'w')
		{
			tone_menu.selected--;
			if(tone_menu.selected < 0)
				tone_menu.selected = 0;
		}
		else if(c == 'd')
		{
			if(tone_menu.selected == 0) //harmonic
			{
				har_num++;
				if(har_num >= NUM_HARMONICS )
				{
					har_num = 0;
				}
			}
			else if(tone_menu.selected == 1) //value
			{
				if(sounds[0].scale[har_num] < MAX_HARMONIC_SCALE)
				{
					for(i=0; i< NUM_STRINGS; i++)
					{
						sounds[i].scale[har_num] += 1;
					}
				}
			}
			else if(tone_menu.selected == 2) //decay
			{
				if(table.decay_const_index[har_num] < SIZE(decay_rates))
				{
					table.decay_const_index[har_num] += 1;
				}
			}
		}
		else if(c == 'a')
		{
			if(tone_menu.selected == 0) //harmonic
			{
				if(har_num == 0)
				{
					har_num = NUM_HARMONICS - 1;
				}
				else
				{
					har_num--;
				}
			}
			else if(tone_menu.selected == 1) //value
			{
				if(sounds[0].scale[har_num] > 0)
				{
					for(i=0; i< NUM_STRINGS; i++)
					{
						sounds[i].scale[har_num] -= 1;
					}
				}
			}
			else if(tone_menu.selected == 2) //decay
			{
				if(table.decay_const_index[har_num] > 0)
				{
					table.decay_const_index[har_num] -= 1;
				}
			}
		}
	}
}

static void _uart_task()
{
	char c;
	int index, i, j;
	char select[MAX_MENU_SIZE][4];

	ansi_code[0] = (char)27;
	ansi_code[1] = 0;
	tone_menu = (menu_t)
	{
		.selected = 0,
		.display = {"Harmonic: ", "Value: ", "Decay: ", "Update Decays ", "Back "},
		.size = 5,
		.child = {NULL},
		.prev = &main_menu,
		.fun = _tone_menu_fun,
	};
	tuning_menu = (menu_t){
			.selected = 0,
			.display = {"Key: ", "Scale: ","Apply", "Back"},
			.size = 4,
			.child = {NULL},
			.prev = &main_menu,
			.fun = _tuning_menu_fun,
	};
	main_menu = (menu_t){
		   .selected = 0,
		   .display = {"Tuning", "Tone"},
		   .size = 2,
		   .child = {&tuning_menu, &tone_menu},
		   .prev = NULL,
		   .fun = NULL
	};

	while(1)
	{
		while(1)
		{
			_print(ansi_code);
			_print("[2J");
			_print("Laser Harp");
			_print(ansi_code);
			_print("E");
			for(i = 0; i < main_menu.size; i++)
			{
				_print(main_menu.display[i]);
				if(main_menu.selected == i)
				{
					_print(" <--");
				}
				else
				{
					_print("      ");
				}
				_print(ansi_code);
				_print("E");
			}
			_print(ansi_code);
			_print("[H");

			c = _read();
			if((c == '\n') || (c == '\r'))
			{
				//TODO: clear uart input buffer here
				(main_menu.child[main_menu.selected])->fun();
			}
			else if(c == 's')
			{
				main_menu.selected++;
				if(main_menu.selected >= main_menu.size)
					main_menu.selected = main_menu.size - 1;
			}
			else if(c == 'w')
			{
				main_menu.selected--;
				if(main_menu.selected < 0)
					main_menu.selected = 0;
			}
		}

	}
}

void sensor_ISR(void * sensor_instance)
{
	BaseType_t xHigherPriorityTaskWoken = 0;
	XGpio* sensor_inst = (XGpio*)sensor_instance;
	XGpio_InterruptDisable(sensor_inst,gpioBTN_INTR_MASK);
	xSemaphoreGiveFromISR(sensor_sem, &xHigherPriorityTaskWoken);
	XGpio_InterruptClear(sensor_inst, gpioBTN_INTR_MASK);
	XGpio_InterruptEnable(sensor_inst, gpioBTN_INTR_MASK);
	portYIELD_FROM_ISR(xHigherPriorityTaskWoken);
}

void button_ISR(void * gpio_instance)
{
	BaseType_t xHigherPriorityTaskWoken = 0;
	XGpio* gpio_inst = (XGpio*)gpio_instance;
	XGpio_InterruptDisable(gpio_inst, gpioBTN_INTR_MASK);
	xSemaphoreGiveFromISR(btn_sem, &xHigherPriorityTaskWoken);
	XGpio_InterruptClear(gpio_inst, gpioBTN_INTR_MASK);
	XGpio_InterruptEnable(gpio_inst, gpioBTN_INTR_MASK);
	portYIELD_FROM_ISR(xHigherPriorityTaskWoken);
}

void dma_s2mm_ISR(void *  XAxiDma_instance)
{
	//do nothing

}

void dma_mm2s_ISR(void *  XAxiDma_instance)
{
	u32 IrqStatus;
	int TimeOut;
	BaseType_t xHigherPriorityTaskWoken = 0;
	XAxiDma *AxiDmaInst = (XAxiDma *)XAxiDma_instance;


	//Read all the pending DMA interrupts
	IrqStatus = XAxiDma_IntrGetIrq(AxiDmaInst, XAXIDMA_DMA_TO_DEVICE);
	//Acknowledge pending interrupts
	XAxiDma_IntrAckIrq(AxiDmaInst, IrqStatus, XAXIDMA_DMA_TO_DEVICE);
	//If there are no interrupts we exit the Handler
	if (!(IrqStatus & XAXIDMA_IRQ_ALL_MASK))
	{
		return;
	}

	// If error interrupt is asserted, raise error flag, reset the
	// hardware to recover from the error, and return with no further
	// processing.
	if (IrqStatus & XAXIDMA_IRQ_ERROR_MASK)
	{
		XAxiDma_Reset(AxiDmaInst);
		TimeOut = 1000;
		while (TimeOut)
		{
			if(XAxiDma_ResetIsDone(AxiDmaInst))
			{
				break;
			}
			TimeOut -= 1;
		}
		return;
	}

	if ((IrqStatus & XAXIDMA_IRQ_IOC_MASK))
	{
		xSemaphoreGiveFromISR(audio_sem, &xHigherPriorityTaskWoken);
		portYIELD_FROM_ISR(xHigherPriorityTaskWoken);
	}
	else return;
}

static void _init_gpio()
{
	int status;
	//init button instane
	status = XGpio_Initialize(&xBtnInstance, gpioDEVICE_ID);
	configASSERT(status == XST_SUCCESS);

	//set button to input
	XGpio_SetDataDirection(&xBtnInstance, 1, 1); //channel one to input

	status = XScuGic_Connect(&xInterruptController, gpioINTC_INTR_ID,
							(Xil_ExceptionHandler)button_ISR, (void*) &xBtnInstance);

	configASSERT(status == XST_SUCCESS);

	XScuGic_Enable(&xInterruptController, gpioINTC_INTR_ID);
	XGpio_InterruptEnable(&xBtnInstance, gpioBTN_INTR_MASK);
	XGpio_InterruptGlobalEnable(&xBtnInstance);

	//sensors
	status = XGpio_Initialize(&xSensorInstance, sensorDEVICE_ID);
	configASSERT(status == XST_SUCCESS);

	//set button to input
	XGpio_SetDataDirection(&xSensorInstance, 1, 1); //channel one to input

	status = XScuGic_Connect(&xInterruptController, sensorINTC_INTR_ID,
							(Xil_ExceptionHandler)sensor_ISR, (void*) &xSensorInstance);

	configASSERT(status == XST_SUCCESS);

	XScuGic_Enable(&xInterruptController, sensorINTC_INTR_ID);
	XGpio_InterruptEnable(&xSensorInstance, gpioBTN_INTR_MASK);
	XGpio_InterruptGlobalEnable(&xSensorInstance);
}

static void _init_audio()
{
	int Status;
	unsigned int i;
	// Initialize IIC controller
	Status = fnInitIic(&sIic);
	configASSERT(Status == XST_SUCCESS);

	//Initialize DMA
	Status = fnConfigDma(&sAxiDma);
	configASSERT(Status == XST_SUCCESS);
	//Initialize Audio I2S
	Status = fnInitAudio();
	configASSERT(Status == XST_SUCCESS);

	{
		XTime  tStart, tEnd;

		XTime_GetTime(&tStart);
		do {
			XTime_GetTime(&tEnd);
		}
		while((tEnd-tStart)/(COUNTS_PER_SECOND/10) < 20);
	}

	Status = fnInitAudio();
	configASSERT(Status == XST_SUCCESS);

	//set up interrupts
	for (i = 0; i < SIZE(_ivt); i++)
	{
		//XScuGic_SetPriorityTriggerType(&xInterruptController, _ivt[i].id, 0xA0, 0x3);  //not sure what this does
		XScuGic_Connect(&xInterruptController, _ivt[i].id, _ivt[i].handler, _ivt[i].pvCallbackRef);
		XScuGic_Enable(&xInterruptController, _ivt[i].id);
	}

	fnSetHpOutput(); //set output to headphone jack
	xTaskCreate( _audio_task,	/* The function that implements the task. */
				 "audio task", 	/* The text name assigned to the task - for debug only as it is not used by the kernel. */
				 1000, 			/* The size of the stack to allocate to the task. */
				 NULL, 			/* The parameter passed to the task - not used in this case. */
				 100, 			/* The priority assigned to the task. */
				 NULL );
}

static void _main_loop()
{
	unsigned int i, j, k;
	volatile int l;
	int32_t str;
	xPort = xSerialPortInitMinimal(115200, 25); //25 because that's what it is in UARTCommandConsole example
	xSemaphoreTake(btn_sem, portMAX_DELAY);
	xSemaphoreTake(sensor_sem, portMAX_DELAY);
	_init_strings();
	_init_audio();
	_init_gpio();
	xTaskCreate( _button_task,		/* The function that implements the task. */
					"button task", 	/* The text name assigned to the task - for debug only as it is not used by the kernel. */
					1000, 			/* The size of the stack to allocate to the task. */
					NULL, 			/* The parameter passed to the task - not used in this case. */
					10, 			/* The priority assigned to the task. */
					NULL );

	xTaskCreate( _sensor_task,		/* The function that implements the task. */
					"button task", 								/* The text name assigned to the task - for debug only as it is not used by the kernel. */
					1000, 			/* The size of the stack to allocate to the task. */
					NULL, 			/* The parameter passed to the task - not used in this case. */
					15, 			/* The priority assigned to the task. */
					NULL );

	xTaskCreate( _uart_task,		/* The function that implements the task. */
					"uart task", 								/* The text name assigned to the task - for debug only as it is not used by the kernel. */
					1000, 			/* The size of the stack to allocate to the task. */
					NULL, 			/* The parameter passed to the task - not used in this case. */
					5, 			/* The priority assigned to the task. */
					NULL );

#define LOOP_INCREMENT  4
#define TIME_INCREMENT  LOOP_INCREMENT / 2
	while(1)
	{
		xSemaphoreTake(calc_sem, portMAX_DELAY);
		for(i = 0; i < BUFFER_SIZE; i += LOOP_INCREMENT)	//for each sample in the buffer
		{
			buffer[buf_num][i] = 0;
			for(j = 0; j < NUM_STRINGS; j++) //for each string
			{
				if(sounds[j].is_active)
				{
					for(k = 0; k < NUM_HARMONICS; k++)  //for each harmonic frequency of that string
					{
						if(sounds[j].scale[k] != 0)
						buffer[buf_num][i] += table.value[j][(sounds[j].time *(k+1)) % table.length[j]] *
							   sounds[j].scale[k] * table.decay[k][sounds[j].time / DECAY_TIME_SCALE];
					}
				}
				sounds[j].time += TIME_INCREMENT;
				if(sounds[j].time > MAX_DECAY * DECAY_TIME_SCALE)
				{
					sounds[j].is_active = 0;
				}
			}
			for(j=1; j< LOOP_INCREMENT; j++)
			{
				buffer[buf_num][i+j] = buffer[buf_num][i];
			}
		}

		Xil_DCacheFlushRange(buffer[buf_num],BUFFER_SIZE*4);
	}
}

int lh_main()
{
	vSemaphoreCreateBinary( btn_sem);
	vSemaphoreCreateBinary(uart_sem);
	vSemaphoreCreateBinary(audio_sem);
	vSemaphoreCreateBinary(calc_sem);
	vSemaphoreCreateBinary(sensor_sem);
	xTaskCreate( _main_loop,			/* The function that implements the task. */
				"main_loop",			/* The text name assigned to the task - for debug only as it is not used by the kernel. */
				1000,					/* The size of the stack to allocate to the task. */
				NULL, 					/* The parameter passed to the task - not used in this case. */
				5, 						/* The priority assigned to the task. */
				NULL );
	vTaskStartScheduler();
}
